@@ -1,10 +1,9 @@
-/* $Id: rmdraw.c,v 1.64 2008/05/21 22:32:40 rmagick Exp $ */
+/* $Id: rmdraw.c,v 1.80 2009/06/19 22:07:40 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2008 by Timothy P. Hunter
+|                Copyright (C) 2009 by Timothy P. Hunter
 | Name:     rmdraw.c
 | Author:   Tim Hunter
 | Purpose:  Contains Draw class methods.
-|           Contains Montage class methods.
 \============================================================================*/
 
 #include "rmagick.h"
@@ -12,7 +11,6 @@
 
 static void mark_Draw(void *);
 static void destroy_Draw(void *);
-static void destroy_Montage(void *);
 static VALUE new_DrawOptions(void);
 
 typedef MagickBooleanType (get_type_metrics_func_t)(Image *, const DrawInfo *, TypeMetric *);
@@ -30,10 +28,11 @@ Draw_affine_eq(VALUE self, VALUE matrix)
 
     rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
-    AffineMatrix_to_AffineMatrix(&draw->info->affine, matrix);
+    Export_AffineMatrix(&draw->info->affine, matrix);
     return self;
 }
 
+
 /*
     Method:     Draw#align=
     Purpose:    set the text alignment
@@ -49,6 +48,7 @@ Draw_align_eq(VALUE self, VALUE align)
     return self;
 }
 
+
 /*
     Method:     Draw#decorate=
     Purpose:    decorate attribute writer
@@ -64,6 +64,7 @@ Draw_decorate_eq(VALUE self, VALUE decorate)
     return self;
 }
 
+
 /*
     Method:     Draw#density=
     Purpose:    density attribute writer
@@ -80,6 +81,7 @@ Draw_density_eq(VALUE self, VALUE density)
     return self;
 }
 
+
 /*
     Method:     Draw#encoding=
     Purpose:    encoding attribute writer
@@ -96,6 +98,7 @@ Draw_encoding_eq(VALUE self, VALUE encoding)
     return self;
 }
 
+
 /*
     Method:     Draw#fill=
     Purpose:    fill attribute writer
@@ -111,6 +114,7 @@ Draw_fill_eq(VALUE self, VALUE fill)
     return self;
 }
 
+
 /*
     Method:     Draw#fill_pattern=
     Purpose:    Accept an image as a fill pattern
@@ -134,7 +138,7 @@ Draw_fill_pattern_eq(VALUE self, VALUE pattern)
 
     if (!NIL_P(pattern))
     {
-        pattern = ImageList_cur_image(pattern);
+        pattern = rm_cur_image(pattern);
         image = rm_check_destroyed(pattern);
         // Do not trace creation
         draw->info->fill_pattern = rm_clone_image(image);
@@ -143,6 +147,7 @@ Draw_fill_pattern_eq(VALUE self, VALUE pattern)
     return self;
 }
 
+
 /*
     Method:     Draw#font=
     Purpose:    font attribute writer
@@ -159,6 +164,7 @@ Draw_font_eq(VALUE self, VALUE font)
     return self;
 }
 
+
 /*
     Method:     Draw#font_family=
     Purpose:    font_family attribute writer
@@ -175,6 +181,7 @@ Draw_font_family_eq(VALUE self, VALUE family)
     return self;
 }
 
+
 /*
     Method:     Draw#font_stretch=
     Purpose:    font_stretch attribute writer
@@ -190,6 +197,7 @@ Draw_font_stretch_eq(VALUE self, VALUE stretch)
     return self;
 }
 
+
 /*
     Method:     Draw#font_style=
     Purpose:    font_style attribute writer
@@ -205,14 +213,13 @@ Draw_font_style_eq(VALUE self, VALUE style)
     return self;
 }
 
+
 /*
     Method:     Draw#font_weight=
     Purpose:    font_weight attribute writer
     Notes:      The font weight can be one of the font weight constants
                 or a number between 100 and 900
 */
-
-
 VALUE
 Draw_font_weight_eq(VALUE self, VALUE weight)
 {
@@ -262,6 +269,7 @@ Draw_font_weight_eq(VALUE self, VALUE weight)
     return self;
 }
 
+
 /*
     Method:     Draw#gravity=
     Purpose:    gravity attribute writer
@@ -289,6 +297,275 @@ Draw_gravity_eq(VALUE self, VALUE grav)
     return self;
 }
 
+
+/*
+    Method:     Draw#kerning=float
+    Purpose:    space between two letters
+    Notes:      new for 6.4.7-8
+*/
+VALUE
+Draw_kerning_eq(VALUE self, VALUE kerning)
+{
+#if defined(HAVE_ST_KERNING)
+    Draw *draw;
+
+    rb_check_frozen(self);
+    Data_Get_Struct(self, Draw, draw);
+    draw->info->kerning = NUM2DBL(kerning);
+    return self;
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+    self = self;
+    kerning = kerning;
+#endif
+}
+
+
+/*
+    Method:     Draw#interword_spacing
+    Purpose:    space between two words
+    Notes:      new for 6.4.8-0
+*/
+VALUE
+Draw_interword_spacing_eq(VALUE self, VALUE spacing)
+{
+#if defined(HAVE_ST_INTERWORD_SPACING)
+    Draw *draw;
+
+    rb_check_frozen(self);
+    Data_Get_Struct(self, Draw, draw);
+    draw->info->interword_spacing = NUM2DBL(spacing);
+    return self;
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+    self = self;
+    spacing = spacing;
+#endif
+}
+
+
+/*
+    Static:     image_to_str
+    Purpose:    Convert an image to a blob and the blob to a String
+    Notes:      Returns Qnil if there is no image
+*/
+static VALUE
+image_to_str(Image *image)
+{
+    volatile VALUE dimg = Qnil;
+    unsigned char *blob;
+    size_t length;
+    Info *info;
+    ExceptionInfo exception;
+
+    if (image)
+    {
+        info = CloneImageInfo(NULL);
+        GetExceptionInfo(&exception);
+        blob = ImageToBlob(info, image, &length, &exception);
+        DestroyImageInfo(info);
+        CHECK_EXCEPTION();
+        DestroyExceptionInfo(&exception);
+        dimg = rb_str_new((char *)blob, (long)length);
+        magick_free((void*)blob);
+    }
+
+    return dimg;
+}
+
+
+/*
+    Static:     str_to_image
+    Purpose:    Undo the image_to_str, above.
+    Notes:      Returns NULL if the argument is Qnil
+*/
+static
+Image *str_to_image(VALUE str)
+{
+    Image *image = NULL;
+    Info *info;
+    ExceptionInfo exception;
+
+    if (str != Qnil)
+    {
+        info = CloneImageInfo(NULL);
+        GetExceptionInfo(&exception);
+        image = BlobToImage(info, RSTRING_PTR(str), RSTRING_LEN(str), &exception);
+        DestroyImageInfo(info);
+        CHECK_EXCEPTION();
+        DestroyExceptionInfo(&exception);
+    }
+
+    return image;
+}
+
+
+/*
+    Method:     Draw#marshal_dump
+    Purpose:    Custom marshal for Draw objects
+    Notes:      Instead of trying to replicate Ruby's support for cross-system
+                marshalling, exploit it. Convert the Draw fields to Ruby objects
+                and store them in a hash. Let Ruby marshal the hash.
+
+                Commented out code that dumps/loads fields that are used internally
+                by ImageMagick and shouldn't be marshaled. I left the code as
+                placeholders so I'll know which fields have been deliberately
+                omitted.
+    To do:      Handle gradients when christy gets the new gradient support added (23Dec08)
+*/
+VALUE
+Draw_marshal_dump(VALUE self)
+{
+    Draw *draw;
+    VALUE ddraw;
+
+    Data_Get_Struct(self, Draw, draw);
+
+    // Raise an exception if the Draw has a non-NULL gradient or element_reference field
+    if (draw->info->element_reference.type != UndefinedReference
+        || draw->info->gradient.type != UndefinedGradient)
+    {
+        rb_raise(rb_eTypeError, "can't dump gradient definition");
+    }
+
+    ddraw = rb_hash_new();
+
+    // rb_hash_aset(ddraw, CSTR2SYM("primitive"), MAGICK_STRING_TO_OBJ(draw->info->primitive)); internal
+    // rb_hash_aset(ddraw, CSTR2SYM("geometry"), MAGICK_STRING_TO_OBJ(draw->info->geometry)); set by "text" primitive
+    // rb_hash_aset(ddraw, CSTR2SYM("viewbox"), Import_RectangleInfo(&draw->info->viewbox)); internal
+    rb_hash_aset(ddraw, CSTR2SYM("affine"), Import_AffineMatrix(&draw->info->affine));
+    rb_hash_aset(ddraw, CSTR2SYM("gravity"), INT2FIX(draw->info->gravity));
+    rb_hash_aset(ddraw, CSTR2SYM("fill"), Pixel_from_PixelPacket(&draw->info->fill));
+    rb_hash_aset(ddraw, CSTR2SYM("stroke"), Pixel_from_PixelPacket(&draw->info->stroke));
+    rb_hash_aset(ddraw, CSTR2SYM("stroke_width"), rb_float_new(draw->info->stroke_width));
+    // rb_hash_aset(ddraw, CSTR2SYM("gradient"), Qnil);  // not used yet
+    rb_hash_aset(ddraw, CSTR2SYM("fill_pattern"), image_to_str(draw->info->fill_pattern));
+    rb_hash_aset(ddraw, CSTR2SYM("tile"), Qnil); // deprecated
+    rb_hash_aset(ddraw, CSTR2SYM("stroke_pattern"), image_to_str(draw->info->stroke_pattern));
+    rb_hash_aset(ddraw, CSTR2SYM("stroke_antialias"), draw->info->stroke_antialias ? Qtrue : Qfalse);
+    rb_hash_aset(ddraw, CSTR2SYM("text_antialias"), draw->info->text_antialias ? Qtrue : Qfalse);
+    // rb_hash_aset(ddraw, CSTR2SYM("fill_rule"), INT2FIX(draw->info->fill_rule)); internal
+    // rb_hash_aset(ddraw, CSTR2SYM("linecap"), INT2FIX(draw->info->linecap));
+    // rb_hash_aset(ddraw, CSTR2SYM("linejoin"), INT2FIX(draw->info->linejoin));
+    // rb_hash_aset(ddraw, CSTR2SYM("miterlimit"), ULONG2NUM(draw->info->miterlimit));
+    // rb_hash_aset(ddraw, CSTR2SYM("dash_offset"), rb_float_new(draw->info->dash_offset));
+    rb_hash_aset(ddraw, CSTR2SYM("decorate"), INT2FIX(draw->info->decorate));
+    // rb_hash_aset(ddraw, CSTR2SYM("compose"), INT2FIX(draw->info->compose)); set via "image" primitive
+    // rb_hash_aset(ddraw, CSTR2SYM("text"), MAGICK_STRING_TO_OBJ(draw->info->text)); set via "text" primitive
+    // rb_hash_aset(ddraw, CSTR2SYM("face"), Qnil);  internal
+    rb_hash_aset(ddraw, CSTR2SYM("font"), MAGICK_STRING_TO_OBJ(draw->info->font));
+    // rb_hash_aset(ddraw, CSTR2SYM("metrics"), Qnil);   internal
+    rb_hash_aset(ddraw, CSTR2SYM("family"), MAGICK_STRING_TO_OBJ(draw->info->family));
+    rb_hash_aset(ddraw, CSTR2SYM("style"), INT2FIX(draw->info->style));
+    rb_hash_aset(ddraw, CSTR2SYM("stretch"), INT2FIX(draw->info->stretch));
+    rb_hash_aset(ddraw, CSTR2SYM("weight"), ULONG2NUM(draw->info->weight));
+    rb_hash_aset(ddraw, CSTR2SYM("encoding"), MAGICK_STRING_TO_OBJ(draw->info->encoding));
+    rb_hash_aset(ddraw, CSTR2SYM("pointsize"), rb_float_new(draw->info->pointsize));
+    rb_hash_aset(ddraw, CSTR2SYM("density"), MAGICK_STRING_TO_OBJ(draw->info->density));
+    rb_hash_aset(ddraw, CSTR2SYM("align"), INT2FIX(draw->info->align));
+    rb_hash_aset(ddraw, CSTR2SYM("undercolor"), Pixel_from_PixelPacket(&draw->info->undercolor));
+    // rb_hash_aset(ddraw, CSTR2SYM("border_color"), Pixel_from_PixelPacket(&draw->info->border_color)); Montage and Polaroid
+    // rb_hash_aset(ddraw, CSTR2SYM("server_name"), MAGICK_STRING_TO_OBJ(draw->info->server_name));
+    // rb_hash_aset(ddraw, CSTR2SYM("dash_pattern"), dash_pattern_to_array(draw->info->dash_pattern)); internal
+    // rb_hash_aset(ddraw, CSTR2SYM("clip_mask"), MAGICK_STRING_TO_OBJ(draw->info->clip_mask)); internal
+    // rb_hash_aset(ddraw, CSTR2SYM("bounds"), Import_SegmentInfo(&draw->info->bounds)); internal
+    rb_hash_aset(ddraw, CSTR2SYM("clip_units"), INT2FIX(draw->info->clip_units));
+    rb_hash_aset(ddraw, CSTR2SYM("opacity"), QUANTUM2NUM(draw->info->opacity));
+    // rb_hash_aset(ddraw, CSTR2SYM("render"), draw->info->render ? Qtrue : Qfalse); internal
+    // rb_hash_aset(ddraw, CSTR2SYM("element_reference"), Qnil);     // not used yet
+    // rb_hash_aset(ddraw, CSTR2SYM("debug"), draw->info->debug ? Qtrue : Qfalse);
+#if defined(HAVE_ST_KERNING)
+    rb_hash_aset(ddraw, CSTR2SYM("kerning"), rb_float_new(draw->info->kerning));
+#endif
+#if defined(HAVE_ST_INTERWORD_SPACING)
+    rb_hash_aset(ddraw, CSTR2SYM("interword_spacing"), rb_float_new(draw->info->interword_spacing));
+#endif
+
+    // Non-DrawInfo fields
+    rb_hash_aset(ddraw, CSTR2SYM("primitives"), draw->primitives);
+    // rb_hash_aset(ddraw, CSTR2SYM("shadow_color"), Pixel_from_PixelPacket(&draw->shadow_color)); Polaroid-only
+
+    return ddraw;
+}
+
+
+/*
+    Method:     Draw#marshal_load
+    Purpose:    Support Marsal.load
+    Notes:      On entry all fields are all-bits-0
+*/
+VALUE
+Draw_marshal_load(VALUE self, VALUE ddraw)
+{
+    Draw *draw;
+    Pixel *pixel;
+    volatile VALUE val;
+
+    Data_Get_Struct(self, Draw, draw);
+
+    draw->info = magick_malloc(sizeof(DrawInfo));
+    if (!draw->info)
+    {
+        rb_raise(rb_eNoMemError, "not enough memory to continue");
+    }
+    GetDrawInfo(NULL, draw->info);
+
+    OBJ_TO_MAGICK_STRING(draw->info->geometry, rb_hash_aref(ddraw, CSTR2SYM("geometry")));
+
+    //val = rb_hash_aref(ddraw, CSTR2SYM("viewbox"));
+    //Export_RectangleInfo(&draw->info->viewbox, val);
+
+    val = rb_hash_aref(ddraw, CSTR2SYM("affine"));
+    Export_AffineMatrix(&draw->info->affine, val);
+
+    draw->info->gravity = (GravityType) FIX2INT(rb_hash_aref(ddraw, CSTR2SYM("gravity")));
+
+    val = rb_hash_aref(ddraw, CSTR2SYM("fill"));
+    Data_Get_Struct(val, Pixel, pixel);
+    draw->info->fill =  *pixel;
+
+    val = rb_hash_aref(ddraw, CSTR2SYM("stroke"));
+    Data_Get_Struct(val, Pixel, pixel);
+    draw->info->stroke = *pixel;
+
+    draw->info->stroke_width = NUM2DBL(rb_hash_aref(ddraw, CSTR2SYM("stroke_width")));
+    draw->info->fill_pattern = str_to_image(rb_hash_aref(ddraw, CSTR2SYM("fill_pattern")));
+    draw->info->stroke_pattern = str_to_image(rb_hash_aref(ddraw, CSTR2SYM("stroke_pattern")));
+    draw->info->stroke_antialias = RTEST(rb_hash_aref(ddraw, CSTR2SYM("stroke_antialias")));
+    draw->info->text_antialias = RTEST(rb_hash_aref(ddraw, CSTR2SYM("text_antialias")));
+    draw->info->decorate = (DecorationType) FIX2INT(rb_hash_aref(ddraw, CSTR2SYM("decorate")));
+    OBJ_TO_MAGICK_STRING(draw->info->font, rb_hash_aref(ddraw, CSTR2SYM("font")));
+    OBJ_TO_MAGICK_STRING(draw->info->family, rb_hash_aref(ddraw, CSTR2SYM("family")));
+
+    draw->info->style = (StyleType) FIX2INT(rb_hash_aref(ddraw, CSTR2SYM("style")));
+    draw->info->stretch = (StretchType) FIX2INT(rb_hash_aref(ddraw, CSTR2SYM("stretch")));
+    draw->info->weight = NUM2ULONG(rb_hash_aref(ddraw, CSTR2SYM("weight")));
+    OBJ_TO_MAGICK_STRING(draw->info->encoding, rb_hash_aref(ddraw, CSTR2SYM("encoding")));
+    draw->info->pointsize = NUM2DBL(rb_hash_aref(ddraw, CSTR2SYM("pointsize")));
+    OBJ_TO_MAGICK_STRING(draw->info->density, rb_hash_aref(ddraw, CSTR2SYM("density")));
+    draw->info->align = (AlignType) FIX2INT(rb_hash_aref(ddraw, CSTR2SYM("align")));
+
+    val = rb_hash_aref(ddraw, CSTR2SYM("undercolor"));
+    Data_Get_Struct(val, Pixel, pixel);
+    draw->info->undercolor = *pixel;
+
+    draw->info->clip_units = FIX2INT(rb_hash_aref(ddraw, CSTR2SYM("clip_units")));
+    draw->info->opacity = NUM2QUANTUM(rb_hash_aref(ddraw, CSTR2SYM("opacity")));
+#if defined(HAVE_ST_KERNING)
+    draw->info->kerning = NUM2DBL(rb_hash_aref(ddraw, CSTR2SYM("kerning")));
+#endif
+#if defined(HAVE_ST_INTERWORD_SPACING)
+    draw->info->interword_spacing = NUM2DBL(rb_hash_aref(ddraw, CSTR2SYM("interword_spacing")));
+#endif
+
+    draw->primitives = rb_hash_aref(ddraw, CSTR2SYM("primitives"));
+
+    return self;
+}
+
+
 /*
     Method:     Draw#pointsize=
     Purpose:    pointsize attribute writer
@@ -304,6 +581,7 @@ Draw_pointsize_eq(VALUE self, VALUE pointsize)
     return self;
 }
 
+
 /*
     Method:     Magick::Draw#rotation=degrees
     Purpose:    set rotation attribute value
@@ -346,6 +624,7 @@ Draw_rotation_eq(VALUE self, VALUE deg)
     return self;
 }
 
+
 /*
     Method:     Draw#stroke=
     Purpose:    stroke attribute writer
@@ -361,6 +640,7 @@ Draw_stroke_eq(VALUE self, VALUE stroke)
     return self;
 }
 
+
 /*
     Method:     Draw#stroke_pattern=
     Purpose:    Accept an image as a stroke pattern
@@ -385,7 +665,7 @@ Draw_stroke_pattern_eq(VALUE self, VALUE pattern)
     if (!NIL_P(pattern))
     {
         // DestroyDrawInfo destroys the clone
-        pattern = ImageList_cur_image(pattern);
+        pattern = rm_cur_image(pattern);
         image = rm_check_destroyed(pattern);
         // Do not trace creation
         draw->info->stroke_pattern = rm_clone_image(image);
@@ -394,6 +674,7 @@ Draw_stroke_pattern_eq(VALUE self, VALUE pattern)
     return self;
 }
 
+
 /*
     Method:     Draw#stroke_width=
     Purpose:    stroke_width attribute writer
@@ -409,6 +690,7 @@ Draw_stroke_width_eq(VALUE self, VALUE stroke_width)
     return self;
 }
 
+
 /*
     Method:     Draw#text_antialias=
     Purpose:    text_antialias attribute writer
@@ -424,6 +706,7 @@ Draw_text_antialias_eq(VALUE self, VALUE text_antialias)
     return self;
 }
 
+
 /*
     Method:     Draw#tile=
     Purpose:    tile attribute writer
@@ -434,6 +717,7 @@ Draw_tile_eq(VALUE self, VALUE image)
     return Draw_fill_pattern_eq(self, image);
 }
 
+
 /*
     Method:     Draw#undercolor=
     Purpose:    undercolor attribute writer
@@ -449,6 +733,7 @@ Draw_undercolor_eq(VALUE self, VALUE undercolor)
     return self;
 }
 
+
 /*
     Method:     Draw#annotate(img, w, h, x, y, text) <{optional parms}>
     Purpose:    annotates an image with text
@@ -478,7 +763,7 @@ VALUE Draw_annotate(
     Data_Get_Struct(self, Draw, draw);
     keep = draw->info->affine;
 
-    image_arg = ImageList_cur_image(image_arg);
+    image_arg = rm_cur_image(image_arg);
     image = rm_check_frozen(image_arg);
 
     // If we have an optional parm block, run it in self's context,
@@ -489,11 +774,7 @@ VALUE Draw_annotate(
     }
 
     // Translate & store in Draw structure
-#if defined(HAVE_INTERPRETIMAGEPROPERTIES)
     draw->info->text = InterpretImageProperties(NULL, image, StringValuePtr(text));
-#else
-    draw->info->text = InterpretImageAttributes(NULL, image, StringValuePtr(text));
-#endif
     if (!draw->info->text)
     {
         rb_raise(rb_eArgError, "no text");
@@ -576,7 +857,7 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     }
 
     // Retrieve the image to composite
-    image = ImageList_cur_image(argv[4]);
+    image = rm_cur_image(argv[4]);
     (void) rm_check_destroyed(image);
 
     x = NUM2DBL(argv[0]);
@@ -597,36 +878,121 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
             case AtopCompositeOp:
                 op = "Atop";
                 break;
+            case BlendCompositeOp:
+                op = "Blend";
+                break;
+#if defined(HAVE_ENUM_BLURCOMPOSITEOP)
+            case BlurCompositeOp:
+                op = "Blur";
+                break;
+#endif
             case BumpmapCompositeOp:
                 op = "Bumpmap";
                 break;
+            case ChangeMaskCompositeOp:
+                op = "ChangeMask";
+                break;
             case ClearCompositeOp:
                 op = "Clear";
                 break;
+            case ColorBurnCompositeOp:
+                op = "ColorBurn";
+                break;
+            case ColorDodgeCompositeOp:
+                op = "ColorDodge";
+                break;
+            case ColorizeCompositeOp:
+                op = "Colorize";
+                break;
+            case CopyCompositeOp:
+                op = "Copy";
+                break;
+            case CopyBlackCompositeOp:
+                op = "CopyBlack";
+                break;
             case CopyBlueCompositeOp:
                 op = "CopyBlue";
                 break;
+            case CopyCyanCompositeOp:
+                op = "CopyCyan";
+                break;
             case CopyGreenCompositeOp:
                 op = "CopyGreen";
                 break;
+            case CopyMagentaCompositeOp:
+                op = "CopyMagenta";
+                break;
             case CopyOpacityCompositeOp:
                 op = "CopyOpacity";
                 break;
             case CopyRedCompositeOp:
                 op = "CopyRed";
                 break;
-            case CopyCompositeOp:
-                op = "Copy";
+            case CopyYellowCompositeOp:
+                op = "CopyYellow";
+                break;
+            case DarkenCompositeOp:
+                op = "Darken";
+                break;
+#if defined(HAVE_ENUM_DISTORTCOMPOSITEOP)
+            case DistortCompositeOp:
+                op = "Distort";
+                break;
+#endif
+            case DivideCompositeOp:
+                op = "Divide";
+                break;
+            case DstCompositeOp:
+                op = "Dst";
+                break;
+            case DstAtopCompositeOp:
+                op = "DstAtop";
+                break;
+            case DstInCompositeOp:
+                op = "DstIn";
+                break;
+            case DstOutCompositeOp:
+                op = "DstOut";
+                break;
+            case DstOverCompositeOp:
+                op = "DstOver";
                 break;
             case DifferenceCompositeOp:
                 op = "Difference";
                 break;
+            case DisplaceCompositeOp:
+                op = "Displace";
+                break;
+            case DissolveCompositeOp:
+                op = "Dissolve";
+                break;
+            case ExclusionCompositeOp:
+                op = "Exclusion";
+                break;
+            case HardLightCompositeOp:
+                op = "HardLight";
+                break;
+            case HueCompositeOp:
+                op = "Hue";
+                break;
             case InCompositeOp:
                 op = "In";
                 break;
+            case LightenCompositeOp:
+                op = "Lighten";
+                break;
+            case LinearLightCompositeOp:
+                op = "LinearLight";
+                break;
+            case LuminizeCompositeOp:
+                op = "Luminize";
+                break;
             case MinusCompositeOp:
                 op = "Minus";
                 break;
+            case ModulateCompositeOp:
+                op = "Modulate";
+                break;
             case MultiplyCompositeOp:
                 op = "Multiply";
                 break;
@@ -636,12 +1002,45 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
             case OverCompositeOp:
                 op = "Over";
                 break;
+            case OverlayCompositeOp:
+                op = "Overlay";
+                break;
             case PlusCompositeOp:
                 op = "Plus";
                 break;
+            case ReplaceCompositeOp:
+                op = "Replace";
+                break;
+            case SaturateCompositeOp:
+                op = "Saturate";
+                break;
+            case ScreenCompositeOp:
+                op = "Screen";
+                break;
+            case SoftLightCompositeOp:
+                op = "SoftLight";
+                break;
+            case SrcCompositeOp:
+                op = "Src";
+                break;
+            case SrcAtopCompositeOp:
+                op = "SrcAtop";
+                break;
+            case SrcInCompositeOp:
+                op = "SrcIn";
+                break;
+            case SrcOutCompositeOp:
+                op = "SrcOut";
+                break;
+            case SrcOverCompositeOp:
+                op = "SrcOver";
+                break;
             case SubtractCompositeOp:
                 op = "Subtract";
                 break;
+            case ThresholdCompositeOp:
+                op = "Threshold";
+                break;
             case XorCompositeOp:
                 op = "Xor";
                 break;
@@ -675,6 +1074,7 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     return self;
 }
 
+
 /*
     Method:     Draw#draw(i)
     Purpose:    Execute the stored drawing primitives on the current image
@@ -686,7 +1086,7 @@ Draw_draw(VALUE self, VALUE image_arg)
     Draw *draw;
     Image *image;
 
-    image_arg = ImageList_cur_image(image_arg);
+    image_arg = rm_cur_image(image_arg);
     image = rm_check_frozen(image_arg);
 
     Data_Get_Struct(self, Draw, draw);
@@ -757,6 +1157,7 @@ Draw_get_multiline_type_metrics(
     return get_type_metrics(argc, argv, self, GetMultilineTypeMetrics);
 }
 
+
 /*
     Method:     Draw#initialize_copy
     Purpose:    initialize clone, dup methods
@@ -803,6 +1204,7 @@ Draw_initialize(VALUE self)
     return self;
 }
 
+
 /*
     Method:     inspect
     Purpose:    display the primitives
@@ -816,12 +1218,12 @@ Draw_inspect(VALUE self)
     return draw->primitives ? draw->primitives : rb_str_new2("(no primitives defined)");
 }
 
+
 /*
     Method:     Draw.new/Draw.allocate
     Purpose:    Create a new Draw object
     Raises:     ImageMagickError if no memory
 */
-
 VALUE Draw_alloc(VALUE class)
 {
     Draw *draw;
@@ -834,6 +1236,7 @@ VALUE Draw_alloc(VALUE class)
     return draw_obj;
 }
 
+
 /*
     Method:     Draw#primitive
     Purpose:    Add a drawing primitive to the list of primitives in the
@@ -860,6 +1263,7 @@ Draw_primitive(VALUE self, VALUE primitive)
     return self;
 }
 
+
 /*
     Static:     mark_Draw
     Purpose:    mark referenced objects
@@ -875,6 +1279,7 @@ mark_Draw(void *drawptr)
     }
 }
 
+
 /*
     Static:     destroy_Draw
     Purpose:    free the memory associated with an Draw object
@@ -908,7 +1313,8 @@ destroy_Draw(void *drawptr)
     Static:     new_DrawOptions
     Purpose:    Allocate & initialize a DrawOptions object.
 */
-static VALUE new_DrawOptions(void)
+static VALUE
+new_DrawOptions(void)
 {
     return DrawOptions_initialize(Draw_alloc(Class_DrawOptions));
 }
@@ -921,7 +1327,8 @@ static VALUE new_DrawOptions(void)
     Notes:      The DrawOptions class is the same as the Draw class except
                 is has only the attribute writer functions.
 */
-VALUE DrawOptions_alloc(VALUE class)
+VALUE
+DrawOptions_alloc(VALUE class)
 {
     Draw *draw_options;
     volatile VALUE draw_options_obj;
@@ -938,7 +1345,8 @@ VALUE DrawOptions_alloc(VALUE class)
     Method:     DrawOptions#initialize
     Purpose:    Initialize a DrawOptions object
 */
-VALUE DrawOptions_initialize(VALUE self)
+VALUE
+DrawOptions_initialize(VALUE self)
 {
     Draw *draw_options;
 
@@ -961,358 +1369,6 @@ VALUE DrawOptions_initialize(VALUE self)
 }
 
 
-/*
-    Method:     Magick::Montage#background_color(color-name)
-    Purpose:    set background_color value
-*/
-VALUE
-Montage_background_color_eq(VALUE self, VALUE color)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    Color_to_PixelPacket(&montage->info->background_color, color);
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#border_color(color-name)
-    Purpose:    set border_color value
-*/
-VALUE
-Montage_border_color_eq(VALUE self, VALUE color)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    Color_to_PixelPacket(&montage->info->border_color, color);
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#border_width(width)
-    Purpose:    set border_width value
-*/
-VALUE
-Montage_border_width_eq(VALUE self, VALUE width)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    montage->info->border_width = NUM2ULONG(width);
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#compose(width)
-    Purpose:    set a composition operator
-*/
-VALUE
-Montage_compose_eq(VALUE self, VALUE compose)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    VALUE_TO_ENUM(compose, montage->compose, CompositeOperator);
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#filename(name)
-    Purpose:    set filename value
-*/
-VALUE
-Montage_filename_eq(VALUE self, VALUE filename)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    strncpy(montage->info->filename, StringValuePtr(filename), MaxTextExtent-1);
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#fill(color-name)
-    Purpose:    set fill value
-*/
-VALUE
-Montage_fill_eq(VALUE self, VALUE color)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    Color_to_PixelPacket(&montage->info->fill, color);
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#font(font-name)
-    Purpose:    set font value
-*/
-VALUE
-Montage_font_eq(VALUE self, VALUE font)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    magick_clone_string(&montage->info->font, StringValuePtr(font));
-
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#frame(frame-geometry)
-    Purpose:    set frame value
-    Notes:      The geometry is a string in the form:
-                <width>x<height>+<outer-bevel-width>+<inner-bevel-width>
-                or a Geometry object
-*/
-VALUE
-Montage_frame_eq(VALUE self, VALUE frame_arg)
-{
-    Montage *montage;
-    volatile VALUE frame;
-
-    Data_Get_Struct(self, Montage, montage);
-    frame = rb_funcall(frame_arg, rm_ID_to_s, 0);
-    magick_clone_string(&montage->info->frame, StringValuePtr(frame));
-
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#geometry(geometry)
-    Purpose:    set geometry value
-*/
-VALUE
-Montage_geometry_eq(VALUE self, VALUE geometry_arg)
-{
-    Montage *montage;
-    volatile VALUE geometry;
-
-    Data_Get_Struct(self, Montage, montage);
-    geometry = rb_funcall(geometry_arg, rm_ID_to_s, 0);
-    magick_clone_string(&montage->info->geometry, StringValuePtr(geometry));
-
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#gravity(gravity-type)
-    Purpose:    set gravity value
-*/
-VALUE
-Montage_gravity_eq(VALUE self, VALUE gravity)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    VALUE_TO_ENUM(gravity, montage->info->gravity, GravityType);
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#initialize
-    Purpose:    Place-holder
-*/
-VALUE
-Montage_initialize(VALUE self)
-{
-    // Nothing to do!
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#matte_color(color-name)
-    Purpose:    set matte_color value
-*/
-VALUE
-Montage_matte_color_eq(VALUE self, VALUE color)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    Color_to_PixelPacket(&montage->info->matte_color, color);
-    return self;
-}
-
-/*
-    Method:     Montage.new
-    Purpose:    Create a new Montage object
-*/
-VALUE
-Montage_alloc(VALUE class)
-{
-    MontageInfo *montage_info;
-    Montage *montage;
-    Info *image_info;
-    volatile VALUE montage_obj;
-
-    // DO NOT call rm_info_new - we don't want to support an Info parm block.
-    image_info = CloneImageInfo(NULL);
-    if (!image_info)
-    {
-        rb_raise(rb_eNoMemError, "not enough memory to initialize Info object");
-    }
-
-    montage_info = CloneMontageInfo(image_info, NULL);
-    (void) (void) DestroyImageInfo(image_info);
-
-    if (!montage_info)
-    {
-        rb_raise(rb_eNoMemError, "not enough memory to initialize Magick::Montage object");
-    }
-
-    montage = ALLOC(Montage);
-    montage->info = montage_info;
-    montage->compose = OverCompositeOp;
-    montage_obj = Data_Wrap_Struct(class, NULL, destroy_Montage, montage);
-
-    return montage_obj;
-}
-
-/*
-    Extern:     rm_montage_new()
-    Purpose:    Return a new Magick::Montage object
-*/
-
-VALUE rm_montage_new(void)
-{
-    return Montage_initialize(Montage_alloc(Class_Montage));
-}
-
-/*
-    Static:     destroy_Montage
-    Purpose:    destory the MontageInfo struct and free the Montage struct
-    Notes:      if the Magick::Montage#texture method wrote a texture file,
-                the file is deleted here.
-*/
-static void
-destroy_Montage(void *obj)
-{
-    Montage *montage = obj;
-
-    // If we saved a temporary texture image, delete it now.
-    if (montage->info && montage->info->texture != NULL)
-    {
-        rm_delete_temp_image(montage->info->texture);
-        magick_free(montage->info->texture);
-        montage->info->texture = NULL;
-    }
-    if (montage->info)
-    {
-        (void) DestroyMontageInfo(montage->info);
-        montage->info = NULL;
-    }
-    xfree(montage);
-}
-
-/*
-    Method:     Magick::Montage#pointsize=size
-    Purpose:    set pointsize value
-*/
-VALUE
-Montage_pointsize_eq(VALUE self, VALUE size)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    montage->info->pointsize = NUM2DBL(size);
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#shadow=shadow
-    Purpose:    set shadow value
-*/
-VALUE
-Montage_shadow_eq(VALUE self, VALUE shadow)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    montage->info->shadow = (MagickBooleanType) RTEST(shadow);
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#stroke(color-name)
-    Purpose:    set stroke value
-*/
-VALUE
-Montage_stroke_eq(VALUE self, VALUE color)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    Color_to_PixelPacket(&montage->info->stroke, color);
-    return self;
-}
-
-/*
-    Method:     Montage#texture(texture-image)
-    Purpose:    set texture value
-*/
-VALUE
-Montage_texture_eq(VALUE self, VALUE texture)
-{
-    Montage *montage;
-    Image *texture_image;
-    char temp_name[MaxTextExtent];
-
-    Data_Get_Struct(self, Montage, montage);
-
-    // If we had a previously defined temp texture image,
-    // remove it now in preparation for this new one.
-    if (montage->info->texture)
-    {
-        rm_delete_temp_image(montage->info->texture);
-        magick_free(montage->info->texture);
-        montage->info->texture = NULL;
-    }
-
-    texture = ImageList_cur_image(texture);
-    texture_image = rm_check_destroyed(texture);
-
-    // Write a temp copy of the image & save its name.
-    rm_write_temp_image(texture_image, temp_name);
-    magick_clone_string(&montage->info->texture, temp_name);
-
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#tile(tile)
-    Purpose:    set tile value
-*/
-VALUE
-Montage_tile_eq(VALUE self, VALUE tile_arg)
-{
-    Montage *montage;
-    volatile VALUE tile;
-
-    Data_Get_Struct(self, Montage, montage);
-    tile = rb_funcall(tile_arg, rm_ID_to_s, 0);
-    magick_clone_string(&montage->info->tile, StringValuePtr(tile));
-
-    return self;
-}
-
-/*
-    Method:     Magick::Montage#title(title)
-    Purpose:    set title value
-*/
-VALUE
-Montage_title_eq(VALUE self, VALUE title)
-{
-    Montage *montage;
-
-    Data_Get_Struct(self, Montage, montage);
-    magick_clone_string(&montage->info->title, StringValuePtr(title));
-    return self;
-}
-
-
 /*
     Extern:     PolaroidOptions_alloc()
     Purpose:    Allocate a new Magick::PolaroidOptions object
@@ -1373,7 +1429,8 @@ PolaroidOptions_initialize(VALUE self)
     Purpose:    allocate a PolaroidOptions instance
     Notes:      Internal use
 */
-VALUE rm_polaroid_new(void)
+VALUE
+rm_polaroid_new(void)
 {
     return PolaroidOptions_initialize(PolaroidOptions_alloc(Class_PolaroidOptions));
 }
@@ -1383,7 +1440,8 @@ VALUE rm_polaroid_new(void)
     Method:     PolaroidOptions#shadow_color=
     Purpose:    Set the shadow color attribute
 */
-VALUE PolaroidOptions_shadow_color_eq(VALUE self, VALUE shadow)
+VALUE
+PolaroidOptions_shadow_color_eq(VALUE self, VALUE shadow)
 {
     Draw *draw;
 
@@ -1397,7 +1455,8 @@ VALUE PolaroidOptions_shadow_color_eq(VALUE self, VALUE shadow)
     Method:     PolaroidOptions#border_color=
     Purpose:    Set the border color attribute
 */
-VALUE PolaroidOptions_border_color_eq(VALUE self, VALUE border)
+VALUE
+PolaroidOptions_border_color_eq(VALUE self, VALUE border)
 {
     Draw *draw;
 
@@ -1407,7 +1466,8 @@ VALUE PolaroidOptions_border_color_eq(VALUE self, VALUE border)
 }
 
 
-static VALUE get_dummy_tm_img(VALUE klass)
+static VALUE
+get_dummy_tm_img(VALUE klass)
 {
 #define DUMMY_IMG_CLASS_VAR "@@_dummy_img_"
     volatile VALUE dummy_img = 0;
@@ -1422,7 +1482,7 @@ static VALUE get_dummy_tm_img(VALUE klass)
         {
             rb_raise(rb_eNoMemError, "not enough memory to continue");
         }
-        image = AllocateImage(info);
+        image = AcquireImage(info);
         if (!image)
         {
             rb_raise(rb_eNoMemError, "not enough memory to continue");
@@ -1489,7 +1549,7 @@ get_type_metrics(
             Data_Get_Struct(get_dummy_tm_img(CLASS_OF(self)), Image, image);
             break;
         case 2:
-            t = ImageList_cur_image(argv[0]);
+            t = rm_cur_image(argv[0]);
             image = rm_check_destroyed(t);
             text = rm_str2cstr(argv[1], &text_l);
             break;                  // okay
@@ -1504,11 +1564,7 @@ get_type_metrics(
     }
 
     Data_Get_Struct(self, Draw, draw);
-#if defined(HAVE_INTERPRETIMAGEPROPERTIES)
     draw->info->text = InterpretImageProperties(NULL, image, text);
-#else
-    draw->info->text = InterpretImageAttributes(NULL, image, text);
-#endif
     if (!draw->info->text)
     {
         rb_raise(rb_eArgError, "no text to measure");
@@ -1527,5 +1583,5 @@ get_type_metrics(
         rb_raise(rb_eRuntimeError, "Can't measure text. Are the fonts installed? "
                  "Is the FreeType library installed?");
     }
-    return TypeMetric_from_TypeMetric(&metrics);
+    return Import_TypeMetric(&metrics);
 }
