@@ -1,6 +1,6 @@
-/* $Id: rmilist.c,v 1.73 2008/07/13 14:31:37 rmagick Exp $ */
+/* $Id: rmilist.c,v 1.92 2009/06/03 23:08:30 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2008 by Timothy P. Hunter
+|                Copyright (C) 2009 by Timothy P. Hunter
 | Name:     rmilist.c
 | Author:   Tim Hunter
 | Purpose:  ImageList class method definitions for RMagick
@@ -11,11 +11,15 @@
 static Image *clone_imagelist(Image *);
 static Image *images_from_imagelist(VALUE);
 static long imagelist_length(VALUE);
+static long check_imagelist_length(VALUE);
 static VALUE imagelist_scene_eq(VALUE, VALUE);
 static void imagelist_push(VALUE, VALUE);
 static VALUE ImageList_new(void);
 
 
+
+
+
 /*
     Method:     ImageList#animate(<delay>)
     Purpose:    repeatedly display the images in the images array to an XWindow
@@ -35,6 +39,9 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
         rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 or 1)", argc);
     }
 
+    // Create a new Info object to use with this call
+    info_obj = rm_info_new();
+
     // Convert the images array to an images sequence.
     images = images_from_imagelist(self);
 
@@ -50,11 +57,7 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
         }
     }
 
-
-    // Create a new Info object to use with this call
-    info_obj = rm_info_new();
     Data_Get_Struct(info_obj, Info, info);
-
     (void) AnimateImages(info, images);
     rm_check_image_exception(images, RetainOnError);
     rm_split(images);
@@ -62,6 +65,7 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
     return self;
 }
 
+
 /*
     Method:     ImageList#append(stack)
     Purpose:    Append all the images by calling ImageAppend
@@ -92,6 +96,7 @@ ImageList_append(VALUE self, VALUE stack_arg)
     return rm_image_new(new_image);
 }
 
+
 /*
     Method:     ImageList#average
     Purpose:    Average all images together by calling AverageImages
@@ -117,6 +122,7 @@ ImageList_average(VALUE self)
     return rm_image_new(new_image);
 }
 
+
 /*
     Method:     ImageList#coalesce
     Purpose:    call CoalesceImages
@@ -154,7 +160,6 @@ ImageList_coalesce(VALUE self)
 VALUE
 ImageList_composite_layers(int argc, VALUE *argv, VALUE self)
 {
-#if defined(HAVE_COMPOSITELAYERS)
     volatile VALUE source_images;
     Image *dest, *source, *new_images;
     RectangleInfo geometry;
@@ -196,16 +201,6 @@ ImageList_composite_layers(int argc, VALUE *argv, VALUE self)
     (void) DestroyExceptionInfo(&exception);
 
     return rm_imagelist_from_images(new_images);
-
-#else
-
-    self = self;
-    argc = argc;
-    argv = argv;
-    rm_not_implemented();
-    return (VALUE)0;
-
-#endif
 }
 
 
@@ -234,6 +229,7 @@ ImageList_deconstruct(VALUE self)
     return rm_imagelist_from_images(new_images);
 }
 
+
 /*
     Method:     ImageList#display
     Purpose:    Display all the images to an X window screen.
@@ -259,6 +255,7 @@ ImageList_display(VALUE self)
     return self;
 }
 
+
 /*
     Method:     ImageList#flatten_images
     Purpose:    merge all the images into a single image
@@ -329,7 +326,6 @@ ImageList_fx(int argc, VALUE *argv, VALUE self)
 }
 
 
-
 /*
     Method:     ImageList#map(reference, dither=false)
     Purpose:    Call MapImages
@@ -344,12 +340,16 @@ ImageList_map(int argc, VALUE *argv, VALUE self)
     volatile VALUE scene, new_imagelist, t;
     ExceptionInfo exception;
 
+#if defined(HAVE_REMAPIMAGES)
+    rb_warning("ImageList#map is deprecated. Use ImageList#remap instead.");
+#endif
+
     switch (argc)
     {
         case 2:
             dither = RTEST(argv[1]);
         case 1:
-            t = ImageList_cur_image(argv[0]);
+            t = rm_cur_image(argv[0]);
             map = rm_check_destroyed(t);
             break;
         default:
@@ -358,11 +358,6 @@ ImageList_map(int argc, VALUE *argv, VALUE self)
     }
 
 
-    if (imagelist_length(self) == 0L)
-    {
-        rb_raise(rb_eArgError, "no images in this image list");
-    }
-
     // Convert image array to image sequence, clone image sequence.
     GetExceptionInfo(&exception);
 
@@ -386,6 +381,7 @@ ImageList_map(int argc, VALUE *argv, VALUE self)
     return new_imagelist;
 }
 
+
 /*
     Method:     ImageList#montage <{parm block}>
     Purpose:    Call MontageImages
@@ -436,6 +432,7 @@ ImageList_montage(VALUE self)
     return rm_imagelist_from_images(new_images);
 }
 
+
 /*
     Method:     ImageList#morph(number_images)
     Purpose:    requires a minimum of two images. The first image is
@@ -451,10 +448,6 @@ ImageList_morph(VALUE self, VALUE nimages)
     ExceptionInfo exception;
     long number_images;
 
-    if (imagelist_length(self) < 1L)
-    {
-        rb_raise(rb_eArgError, "no images in this image list");
-    }
 
     // Use a signed long so we can test for a negative argument.
     number_images = NUM2LONG(nimages);
@@ -475,6 +468,7 @@ ImageList_morph(VALUE self, VALUE nimages)
     return rm_imagelist_from_images(new_images);
 }
 
+
 /*
     Method:     ImageList#mosaic
     Purpose:    merge all the images into a single image
@@ -535,30 +529,22 @@ ImageList_optimize_layers(VALUE self, VALUE method)
         case DisposeLayer:
             new_images = DisposeImages(images, &exception);
             break;
-#if defined(HAVE_ENUM_OPTIMIZETRANSLAYER)
         case OptimizeTransLayer:
             new_images = clone_imagelist(images);
             OptimizeImageTransparency(new_images, &exception);
             break;
-#endif
-#if defined(HAVE_ENUM_REMOVEDUPSLAYER)
         case RemoveDupsLayer:
             new_images = clone_imagelist(images);
             RemoveDuplicateLayers(&new_images, &exception);
             break;
-#endif
-#if defined(HAVE_ENUM_REMOVEZEROLAYER)
         case RemoveZeroLayer:
             new_images = clone_imagelist(images);
             RemoveZeroDelayLayers(&new_images, &exception);
             break;
-#endif
-#if defined(HAVE_ENUM_COMPOSITELAYER)
         case CompositeLayer:
+            rm_split(images);
             rb_raise(rb_eNotImpError, "Magick::CompositeLayer is not supported. Use the composite_layers method instead.");
             break;
-#endif
-#if defined(HAVE_ENUM_OPTIMIZEIMAGELAYER)
             // In 6.3.4-ish, OptimizeImageLayer replaced OptimizeLayer
         case OptimizeImageLayer:
             new_images = OptimizeImageLayers(images, &exception);
@@ -577,11 +563,6 @@ ImageList_optimize_layers(VALUE self, VALUE method)
             // mogrify supports -dither here. We don't.
             (void) MapImages(new_images, NULL, 0);
             break;
-#else
-        case OptimizeLayer:
-            new_images = OptimizeImageLayers(images, &exception);
-            break;
-#endif
         case OptimizePlusLayer:
             new_images = OptimizePlusImageLayers(images, &exception);
             break;
@@ -604,8 +585,14 @@ ImageList_optimize_layers(VALUE self, VALUE method)
         case MergeLayer:
             new_images = MergeImageLayers(images, mthd, &exception);
             break;
+#endif
+#if defined(HAVE_ENUM_TRIMBOUNDSLAYER)
+        case TrimBoundsLayer:
+            new_images = MergeImageLayers(images, mthd, &exception);
+            break;
 #endif
         default:
+            rm_split(images);
             rb_raise(rb_eArgError, "undefined layer method");
             break;
     }
@@ -674,11 +661,7 @@ images_from_imagelist(VALUE imagelist)
     Image *head = NULL;
     volatile VALUE images, t;
 
-    len = imagelist_length(imagelist);
-    if (len == 0)
-    {
-        rb_raise(rb_eArgError, "no images in this image list");
-    }
+    len = check_imagelist_length(imagelist);
 
     images = rb_iv_get(imagelist, "@images");
     for (x = 0; x < len; x++)
@@ -706,6 +689,7 @@ imagelist_scene_eq(VALUE imagelist, VALUE scene)
     return scene;
 }
 
+
 /*
     Static:    imagelist_length
     Purpose:   return the # of images in an imagelist
@@ -717,6 +701,25 @@ imagelist_length(VALUE imagelist)
     return RARRAY_LEN(images);
 }
 
+
+/*
+    Static:    check_imagelist_length
+    Purpose:   raise exception if imagelist is emtpy
+*/
+static long
+check_imagelist_length(VALUE imagelist)
+{
+    long len = imagelist_length(imagelist);
+
+    if (len == 0)
+    {
+        rb_raise(rb_eArgError, "no images in this image list");
+    }
+
+    return len;
+}
+
+
 /*
     Static:     imagelist_push
     Purpose:    push an image onto the end of the imagelist
@@ -780,7 +783,15 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
         case 4:
             quantize_info.tree_depth = (unsigned long)NUM2INT(argv[3]);
         case 3:
+#if defined(HAVE_TYPE_DITHERMETHOD) && defined(HAVE_ENUM_NODITHERMETHOD)
+            if (rb_obj_is_kind_of(argv[2], Class_DitherMethod))
+            {
+                VALUE_TO_ENUM(argv[2], quantize_info.dither_method, DitherMethod);
+                quantize_info.dither = quantize_info.dither_method != NoDitherMethod;
+            }
+#else
             quantize_info.dither = (MagickBooleanType) RTEST(argv[2]);
+#endif
         case 2:
             VALUE_TO_ENUM(argv[1], quantize_info.colorspace, ColorspaceType);
         case 1:
@@ -792,10 +803,6 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
             break;
     }
 
-    if (imagelist_length(self) == 0L)
-    {
-        rb_raise(rb_eArgError, "no images in this image list");
-    }
 
     // Convert image array to image sequence, clone image sequence.
     GetExceptionInfo(&exception);
@@ -826,6 +833,59 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
     return new_imagelist;
 }
 
+
+/*
+    Method:     ImageList#remap(remap_image=nil, dither_method=RiemersmaDitherMethod)
+    Purpose:    Call RemapImages
+    Note:       See Image_remap. Immediate - modifies images in-place
+*/
+VALUE
+ImageList_remap(int argc, VALUE *argv, VALUE self)
+{
+#if defined(HAVE_REMAPIMAGES) || defined(HAVE_AFFINITYIMAGES)
+    Image *images, *remap_image = NULL;
+    QuantizeInfo quantize_info;
+
+
+    if (argc > 0 && argv[0] != Qnil)
+    {
+        volatile VALUE t = rm_cur_image(argv[0]);
+        remap_image = rm_check_destroyed(t);
+    }
+
+    GetQuantizeInfo(&quantize_info);
+
+    if (argc > 1)
+    {
+        VALUE_TO_ENUM(argv[1], quantize_info.dither_method, DitherMethod);
+        quantize_info.dither = MagickTrue;
+    }
+    if (argc > 2)
+    {
+        rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
+    }
+
+    images = images_from_imagelist(self);
+
+#if defined(HAVE_REMAPIMAGES)
+    (void) RemapImages(&quantize_info, images, remap_image);
+#else
+    (void) AffinityImages(&quantize_info, images, remap_image);
+#endif
+    rm_check_image_exception(images, RetainOnError);
+    rm_split(images);
+
+    return self;
+#else
+    self = self;
+    argc = argc;
+    argv = argv;
+    rm_not_implemented();
+    return(VALUE)0;
+#endif
+}
+
+
 /*
     Method:     ImageList#to_blob
     Purpose:    returns the imagelist as a blob (a String)
@@ -835,7 +895,7 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
 VALUE
 ImageList_to_blob(VALUE self)
 {
-    Image *images;
+    Image *images, *image;
     Info *info;
     volatile VALUE info_obj;
     volatile VALUE blob_str;
@@ -862,14 +922,21 @@ ImageList_to_blob(VALUE self)
         }
     }
 
+    for (image = images; image; image = GetNextImageInList(image))
+    {
+        rm_sync_image_options(image, info);
+    }
+
     // Unconditionally request multi-images support. The worst that
     // can happen is that there's only one image or the format
     // doesn't support multi-image files.
     info->adjoin = MagickTrue;
     blob = ImagesToBlob(info, images, &length, &exception);
-    if (exception.severity != UndefinedException)
+    if (blob && exception.severity >= ErrorException)
     {
         magick_free((void*)blob);
+        blob = NULL;
+        length = 0;
     }
     rm_split(images);
     CHECK_EXCEPTION()
@@ -888,17 +955,6 @@ ImageList_to_blob(VALUE self)
 }
 
 
-/*
- *  Static:     file_arg_rescue
- *  Purpose:    called when `arg_to_str' raised an exception
-*/
-static VALUE file_arg_rescue(VALUE arg)
-{
-    rb_raise(rb_eTypeError, "argument must be path name or open file (%s given)",
-             rb_class2name(CLASS_OF(arg)));
-}
-
-
 /*
   Method:   ImageList#write(file)
   Purpose:  Write all the images to the specified file. If the file format
@@ -914,8 +970,6 @@ ImageList_write(VALUE self, VALUE file)
     Info *info;
     const MagickInfo *m;
     volatile VALUE info_obj;
-    char *filename;
-    long filename_l;
     unsigned long scene;
     ExceptionInfo exception;
 
@@ -933,21 +987,14 @@ ImageList_write(VALUE self, VALUE file)
     }
     else
     {
-        // Convert arg to string. Catch exceptions.
-        file = rb_rescue(rb_String, file, file_arg_rescue, file);
-
-        // Copy the filename to the Info and to the Image.
-        filename = rm_str2cstr(file, &filename_l);
-        filename_l = min(filename_l, MaxTextExtent-1);
-        memcpy(info->filename, filename, (size_t)filename_l);
-        info->filename[filename_l] = '\0';
+        add_format_prefix(info, file);
         SetImageInfoFile(info, NULL);
     }
 
     // Convert the images array to an images sequence.
     images = images_from_imagelist(self);
 
-    // Copy the filename into each images. Set a scene number to be used if
+    // Copy the filename into each image. Set a scene number to be used if
     // writing multiple files. (Ref: ImageMagick's utilities/convert.c
     for (scene = 0, img = images; img; img = GetNextImageInList(img))
     {
@@ -955,13 +1002,11 @@ ImageList_write(VALUE self, VALUE file)
         strcpy(img->filename, info->filename);
     }
 
+    // Find out if the format supports multi-images files.
     GetExceptionInfo(&exception);
     (void) SetImageInfo(info, MagickTrue, &exception);
     rm_check_exception(&exception, images, RetainOnError);
-    (void) DestroyExceptionInfo(&exception);
 
-    // Find out if the format supports multi-images files.
-    GetExceptionInfo(&exception);
     m = GetMagickInfo(info->magick, &exception);
     rm_check_exception(&exception, images, RetainOnError);
     (void) DestroyExceptionInfo(&exception);
@@ -974,6 +1019,7 @@ ImageList_write(VALUE self, VALUE file)
 
     for (img = images; img; img = GetNextImageInList(img))
     {
+        rm_sync_image_options(img, info);
         (void) WriteImage(info, img);
         // images will be split before raising an exception
         rm_check_image_exception(images, RetainOnError);
