@@ -1,6 +1,6 @@
-/* $Id: rmmain.c,v 1.252 2008/07/13 21:18:28 rmagick Exp $ */
+/* $Id: rmmain.c,v 1.296 2009/06/19 22:07:05 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2008 by Timothy P. Hunter
+|                Copyright (C) 2009 by Timothy P. Hunter
 | Name:     rmmain.c
 | Author:   Tim Hunter
 | Purpose:  Contains all module, class, method declarations.
@@ -22,300 +22,71 @@ static void version_constants(void);
 
 
 
+
 /*
-    Method:     Magick::colors [ { |colorinfo| } ]
-    Purpose:    If called with the optional block, iterates over the colors,
-                otherwise returns an array of Magick::Color objects
-    Notes:      There are 3 implementations
-
-*/
-VALUE
-Magick_colors(VALUE class)
+ *  Handle transferring ImageMagick memory allocations/frees to Ruby.
+ *  These functions have the same signature as the equivalent C functions.
+ */
+#if defined(HAVE_SETMAGICKMEMORYMETHODS)
+static void *rm_malloc(size_t size)
 {
-    const ColorInfo **color_info_list;
-    unsigned long number_colors, x;
-    volatile VALUE ary;
-    ExceptionInfo exception;
+    void *p;
+//    int old_state;
 
-    GetExceptionInfo(&exception);
+//    old_state = rb_gc_disable();
+    p = xmalloc((long)size);
+//    if (!RTEST(old_state))
+//    {
+//        rb_gc_enable();
+//    }
 
-    color_info_list = GetColorInfoList("*", &number_colors, &exception);
-    CHECK_EXCEPTION()
-    (void) DestroyExceptionInfo(&exception);
+    return p;
+}
 
 
-    if (rb_block_given_p())
+
+
+static void *rm_realloc(void *ptr, size_t size)
+{
+    void *p;
+//    int old_state;
+
+//    old_state = rb_gc_disable();
+    p = xrealloc(ptr, (long)size);
+//    if (!RTEST(old_state))
+//    {
+//        rb_gc_enable();
+//    }
+
+    return p;
+}
+
+
+
+
+static void rm_free(void *ptr)
+{
+    xfree(ptr);
+}
+
+
+static void set_managed_memory(void)
+{
+    ID enable_mm = rb_intern("RMAGICK_ENABLE_MANAGED_MEMORY");
+
+    if (RTEST(rb_const_defined(rb_cObject, enable_mm)) && RTEST(rb_const_get(rb_cObject, enable_mm)))
     {
-        for (x = 0; x < number_colors; x++)
-        {
-            (void) rb_yield(Color_from_ColorInfo(color_info_list[x]));
-        }
-        magick_free((void *)color_info_list);
-        return class;
+        rb_warning("RMagick: %s", "managed memory enabled. This is an experimental feature.");
+        SetMagickMemoryMethods(rm_malloc, rm_realloc, rm_free);
+        rb_define_const(Module_Magick, "MANAGED_MEMORY", Qtrue);
     }
     else
     {
-        ary = rb_ary_new2((long) number_colors);
-        for (x = 0; x < number_colors; x++)
-        {
-            (void) rb_ary_push(ary, Color_from_ColorInfo(color_info_list[x]));
-        }
-
-        magick_free((void *)color_info_list);
-        return ary;
+        rb_define_const(Module_Magick, "MANAGED_MEMORY", Qfalse);
     }
 }
+#endif
 
-/*
-  Method:   Magick::fonts [ { |fontinfo| } ]
-  Purpose:  If called with the optional block, iterates over the fonts,
-            otherwise returns an array of Magick::Font objects
-*/
-VALUE
-Magick_fonts(VALUE class)
-{
-    const TypeInfo **type_info;
-    unsigned long number_types, x;
-    volatile VALUE ary;
-    ExceptionInfo exception;
-
-    GetExceptionInfo(&exception);
-    type_info = GetTypeInfoList("*", &number_types, &exception);
-    CHECK_EXCEPTION()
-    (void) DestroyExceptionInfo(&exception);
-
-    if (rb_block_given_p())
-    {
-        for (x = 0; x < number_types; x++)
-        {
-            (void) rb_yield(Font_from_TypeInfo((const TypeInfo *)type_info[x]));
-        }
-        magick_free((void *)type_info);
-        return class;
-    }
-    else
-    {
-        ary = rb_ary_new2((long)number_types);
-        for (x = 0; x < number_types; x++)
-        {
-            (void) rb_ary_push(ary, Font_from_TypeInfo((const TypeInfo *)type_info[x]));
-        }
-        magick_free((void *)type_info);
-        return ary;
-    }
-
-}
-
-
-/*
-  Method:   Magick.init_formats
-  Purpose:  Build the @@formats hash
-
-            The hash keys are image formats. The hash values
-            specify the format "mode string", i.e. a description of what
-            ImageMagick can do with that format. The mode string is in the
-            form "BRWA", where
-                "B" is "*" if the format has native blob support, or " " otherwise.
-                "R" is "r" if ImageMagick can read that format, or "-" otherwise.
-                "W" is "w" if ImageMagick can write that format, or "-" otherwise.
-                "A" is "+" if the format supports multi-image files, or "-" otherwise.
-  Notes:    Only called once.
-            There are 3 implementations.
-*/
-
-static VALUE MagickInfo_to_format(const MagickInfo *magick_info)
-{
-    char mode[4];
-
-    mode[0] = magick_info->blob_support ? '*': ' ';
-    mode[1] = magick_info->decoder ? 'r' : '-';
-    mode[2] = magick_info->encoder ? 'w' : '-';
-    mode[3] = magick_info->encoder && magick_info->adjoin ? '+' : '-';
-
-    return rb_str_new(mode, sizeof(mode));
-}
-
-VALUE
-Magick_init_formats(VALUE class)
-{
-    const MagickInfo **magick_info;
-    unsigned long number_formats, x;
-    volatile VALUE formats;
-    ExceptionInfo exception;
-
-    class = class;      // defeat "never referenced" message from icc
-    formats = rb_hash_new();
-
-    // IM 6.1.3 added an exception argument to GetMagickInfoList
-    GetExceptionInfo(&exception);
-    magick_info = GetMagickInfoList("*", &number_formats, &exception);
-    CHECK_EXCEPTION()
-    (void) DestroyExceptionInfo(&exception);
-
-
-    for (x = 0; x < number_formats; x++)
-    {
-        (void) rb_hash_aset(formats
-                            , rb_str_new2(magick_info[x]->name)
-                            , MagickInfo_to_format((const MagickInfo *)magick_info[x]));
-    }
-    return formats;
-}
-
-
-/*
-  Method:   Magick.limit_resource(resource[, limit])
-  Purpose:  Get/set resource limits. If a limit is specified the old limit
-            is set to the new value. Either way the current/new limit is returned.
-*/
-static VALUE
-Magick_limit_resource(int argc, VALUE *argv, VALUE class)
-{
-    volatile VALUE resource, limit;
-    ResourceType res = UndefinedResource;
-    char *str;
-    ID id;
-    unsigned long cur_limit;
-
-    rb_scan_args(argc, argv, "11", &resource, &limit);
-
-    switch (TYPE(resource))
-    {
-        case T_NIL:
-            return class;
-
-        case T_SYMBOL:
-            id = (ID)SYM2ID(resource);
-            if (id == rb_intern("area"))
-            {
-                res = AreaResource;
-            }
-            else if (id == rb_intern("memory"))
-            {
-                res = MemoryResource;
-            }
-            else if (id == rb_intern("map"))
-            {
-                res = MapResource;
-            }
-            else if (id == rb_intern("disk"))
-            {
-                res = DiskResource;
-            }
-            else if (id == rb_intern("file"))
-            {
-                res = FileResource;
-            }
-            else
-            {
-                rb_raise(rb_eArgError, "unknown resource: `:%s'", rb_id2name(id));
-            }
-            break;
-
-        default:
-            str = StringValuePtr(resource);
-            if (*str == '\0')
-            {
-                return class;
-            }
-            else if (rm_strcasecmp("area", str) == 0)
-            {
-                res = AreaResource;
-            }
-            else if (rm_strcasecmp("memory", str) == 0)
-            {
-                res = MemoryResource;
-            }
-            else if (rm_strcasecmp("map", str) == 0)
-            {
-                res = MapResource;
-            }
-            else if (rm_strcasecmp("disk", str) == 0)
-            {
-                res = DiskResource;
-            }
-            else if (rm_strcasecmp("file", str) == 0)
-            {
-                res = FileResource;
-            }
-            else
-            {
-                rb_raise(rb_eArgError, "unknown resource: `%s'", str);
-            }
-            break;
-    }
-
-    cur_limit = GetMagickResourceLimit(res);
-
-    if (argc > 1)
-    {
-        (void) SetMagickResourceLimit(res, (MagickSizeType)NUM2ULONG(limit));
-    }
-
-    return ULONG2NUM(cur_limit);
-}
-
-
-
-/*
-    Method      Magick.set_cache_threshold(megabytes)
-    Purpose:    sets the amount of free memory allocated for the
-                pixel cache.  Once this threshold is exceeded, all
-                subsequent pixels cache operations are to/from disk.
-    Notes:      singleton method
-*/
-static VALUE
-Magick_set_cache_threshold(VALUE class, VALUE threshold)
-{
-    unsigned long thrshld = NUM2ULONG(threshold);
-    (void) SetMagickResourceLimit(MemoryResource, (MagickSizeType)thrshld);
-    (void) SetMagickResourceLimit(MapResource, (MagickSizeType)(2*thrshld));
-    return class;
-}
-
-/*
-    Method:     Magick.set_log_event_mask(event,...) -> Magick
-    Notes:      "event" is one of "all", "annotate", "blob", "cache",
-                "coder", "configure", "deprecate", "locale", "none",
-                "render", "transform", "user", "x11". Multiple events
-                can be specified. Event names may be capitalized.
-*/
-static VALUE
-Magick_set_log_event_mask(int argc, VALUE *argv, VALUE class)
-{
-    int x;
-
-    if (argc == 0)
-    {
-        rb_raise(rb_eArgError, "wrong number of arguments (at least 1 required)");
-    }
-    for (x = 0; x < argc; x++)
-    {
-        (void) SetLogEventMask(StringValuePtr(argv[x]));
-    }
-    return class;
-}
-
-/*
-    Method:     Magick.set_log_format(format) -> Magick
-    Notes:      Format is a string containing one or more of:
-                %t  - current time
-                %r  - elapsed time
-                %u  - user time
-                %p  - pid
-                %m  - module (source file name)
-                %f  - function name
-                %l  - line number
-                %d  - event domain (one of the events listed above)
-                %e  - event name
-                Plus other characters, including \n, etc.
-*/
-static VALUE
-Magick_set_log_format(VALUE class, VALUE format)
-{
-    SetLogFormat(StringValuePtr(format));
-    return class;
-}
 
 
 
@@ -325,22 +96,23 @@ Magick_set_log_format(VALUE class, VALUE format)
   Arguments:    void
   Returns:      void
 */
-
 void
 Init_RMagick2(void)
 {
     volatile VALUE observable;
 
-#if defined(HAVE_MAGICKCOREGENESIS)
     MagickCoreGenesis("RMagick", MagickFalse);
-#else
-    InitializeMagick("RMagick");
-#endif
 
     test_Magick_version();
 
     Module_Magick = rb_define_module("Magick");
 
+#if defined(HAVE_SETMAGICKMEMORYMETHODS)
+    set_managed_memory();
+#else
+    rb_define_const(Module_Magick, "MANAGED_MEMORY", Qfalse);
+#endif
+
     /*-----------------------------------------------------------------------*/
     /* Create IDs for frequently used methods, etc.                          */
     /*-----------------------------------------------------------------------*/
@@ -355,6 +127,7 @@ Init_RMagick2(void)
     rm_ID_from_s           = rb_intern("from_s");
     rm_ID_Geometry         = rb_intern("Geometry");
     rm_ID_GeometryValue    = rb_intern("GeometryValue");
+    rm_ID_has_key_q        = rb_intern("has_key?");
     rm_ID_height           = rb_intern("height");
     rm_ID_initialize_copy  = rb_intern("initialize_copy");
     rm_ID_length           = rb_intern("length");
@@ -462,6 +235,7 @@ Init_RMagick2(void)
     DCL_ATTR_ACCESSOR(Image, class_type)
     DCL_ATTR_ACCESSOR(Image, ticks_per_second)
     DCL_ATTR_READER(Image, total_colors)
+    DCL_ATTR_READER(Image, total_ink_density)
     DCL_ATTR_ACCESSOR(Image, transparent_color)
     DCL_ATTR_ACCESSOR(Image, units)
     DCL_ATTR_ACCESSOR(Image, virtual_pixel_method)
@@ -479,6 +253,7 @@ Init_RMagick2(void)
     rb_define_method(Class_Image, "add_noise_channel", Image_add_noise_channel, -1);
     rb_define_method(Class_Image, "add_profile", Image_add_profile, 1);
     rb_define_method(Class_Image, "affine_transform", Image_affine_transform, 1);
+    rb_define_method(Class_Image, "remap", Image_remap, -1);
     rb_define_method(Class_Image, "alpha", Image_alpha, -1);
     rb_define_method(Class_Image, "alpha?", Image_alpha_q, 0);
     rb_define_method(Class_Image, "[]", Image_aref, 1);
@@ -517,6 +292,8 @@ Init_RMagick2(void)
     rb_define_method(Class_Image, "composite_affine", Image_composite_affine, 2);
     rb_define_method(Class_Image, "composite_channel", Image_composite_channel, -1);
     rb_define_method(Class_Image, "composite_channel!", Image_composite_channel_bang, -1);
+    rb_define_method(Class_Image, "composite_tiled", Image_composite_tiled, -1);
+    rb_define_method(Class_Image, "composite_tiled!", Image_composite_tiled_bang, -1);
     rb_define_method(Class_Image, "compress_colormap!", Image_compress_colormap_bang, 0);
     rb_define_method(Class_Image, "contrast", Image_contrast, -1);
     rb_define_method(Class_Image, "contrast_stretch_channel", Image_contrast_stretch_channel, -1);
@@ -527,6 +304,8 @@ Init_RMagick2(void)
     rb_define_method(Class_Image, "crop!", Image_crop_bang, -1);
     rb_define_method(Class_Image, "cycle_colormap", Image_cycle_colormap, 1);
     rb_define_method(Class_Image, "decipher", Image_decipher, 1);
+    rb_define_method(Class_Image, "define", Image_define, 2);
+    rb_define_method(Class_Image, "deskew", Image_deskew, -1);
     rb_define_method(Class_Image, "delete_compose_mask", Image_delete_compose_mask, 0);
     rb_define_method(Class_Image, "delete_profile", Image_delete_profile, 1);
     rb_define_method(Class_Image, "despeckle", Image_despeckle, 0);
@@ -574,11 +353,15 @@ Init_RMagick2(void)
     rb_define_method(Class_Image, "inspect", Image_inspect, 0);
     rb_define_method(Class_Image, "level2", Image_level2, -1);
     rb_define_method(Class_Image, "level_channel", Image_level_channel, -1);
+    rb_define_method(Class_Image, "level_colors", Image_level_colors, -1);
+    rb_define_method(Class_Image, "levelize_channel", Image_levelize_channel, -1);
     rb_define_method(Class_Image, "linear_stretch", Image_linear_stretch, -1);
     rb_define_method(Class_Image, "liquid_rescale", Image_liquid_rescale, -1);
     rb_define_method(Class_Image, "magnify", Image_magnify, 0);
     rb_define_method(Class_Image, "magnify!", Image_magnify_bang, 0);
     rb_define_method(Class_Image, "map", Image_map, -1);
+    rb_define_method(Class_Image, "marshal_dump", Image_marshal_dump, 0);
+    rb_define_method(Class_Image, "marshal_load", Image_marshal_load, 1);
     rb_define_method(Class_Image, "mask", Image_mask, -1);
     rb_define_method(Class_Image, "matte_flood_fill", Image_matte_flood_fill, 5);
     rb_define_method(Class_Image, "median_filter", Image_median_filter, -1);
@@ -622,6 +405,7 @@ Init_RMagick2(void)
     rb_define_method(Class_Image, "scale", Image_scale, -1);
     rb_define_method(Class_Image, "scale!", Image_scale_bang, -1);
     rb_define_method(Class_Image, "segment", Image_segment, -1);
+    rb_define_method(Class_Image, "selective_blur_channel", Image_selective_blur_channel, -1);
     rb_define_method(Class_Image, "separate", Image_separate, -1);
     rb_define_method(Class_Image, "sepiatone", Image_sepiatone, -1);
     rb_define_method(Class_Image, "set_channel_depth", Image_set_channel_depth, 2);
@@ -637,6 +421,7 @@ Init_RMagick2(void)
     rb_define_method(Class_Image, "sketch", Image_sketch, -1);
     rb_define_method(Class_Image, "solarize", Image_solarize, -1);
     rb_define_method(Class_Image, "<=>", Image_spaceship, 1);
+    rb_define_method(Class_Image, "sparse_color", Image_sparse_color, -1);
     rb_define_method(Class_Image, "splice", Image_splice, -1);
     rb_define_method(Class_Image, "spread", Image_spread, -1);
     rb_define_method(Class_Image, "stegano", Image_stegano, 2);
@@ -653,12 +438,14 @@ Init_RMagick2(void)
     rb_define_method(Class_Image, "to_color", Image_to_color, 1);
     rb_define_method(Class_Image, "to_blob", Image_to_blob, 0);
     rb_define_method(Class_Image, "transparent", Image_transparent, -1);
+    rb_define_method(Class_Image, "transparent_chroma", Image_transparent_chroma, -1);
     rb_define_method(Class_Image, "transpose", Image_transpose, 0);
     rb_define_method(Class_Image, "transpose!", Image_transpose_bang, 0);
     rb_define_method(Class_Image, "transverse", Image_transverse, 0);
     rb_define_method(Class_Image, "transverse!", Image_transverse_bang, 0);
     rb_define_method(Class_Image, "trim", Image_trim, -1);
     rb_define_method(Class_Image, "trim!", Image_trim_bang, -1);
+    rb_define_method(Class_Image, "undefine", Image_undefine, 1);
     rb_define_method(Class_Image, "unique_colors", Image_unique_colors, 0);
     rb_define_method(Class_Image, "unsharp_mask", Image_unsharp_mask, -1);
     rb_define_method(Class_Image, "unsharp_mask_channel", Image_unsharp_mask_channel, -1);
@@ -677,6 +464,7 @@ Init_RMagick2(void)
 
     // Define an alias for Object#display before we override it
     rb_define_alias(Class_ImageList, "__display__", "display");
+    rb_define_method(Class_ImageList, "remap", ImageList_remap, -1);
     rb_define_method(Class_ImageList, "animate", ImageList_animate, -1);
     rb_define_method(Class_ImageList, "append", ImageList_append, 1);
     rb_define_method(Class_ImageList, "average", ImageList_average, 0);
@@ -715,6 +503,8 @@ Init_RMagick2(void)
     DCL_ATTR_WRITER(Draw, font_style)
     DCL_ATTR_WRITER(Draw, font_weight)
     DCL_ATTR_WRITER(Draw, gravity)
+    DCL_ATTR_WRITER(Draw, interword_spacing)
+    DCL_ATTR_WRITER(Draw, kerning)
     DCL_ATTR_WRITER(Draw, pointsize)
     DCL_ATTR_WRITER(Draw, rotation)
     DCL_ATTR_WRITER(Draw, stroke)
@@ -734,6 +524,8 @@ Init_RMagick2(void)
     rb_define_method(Class_Draw, "initialize", Draw_initialize, 0);
     rb_define_method(Class_Draw, "initialize_copy", Draw_init_copy, 1);
     rb_define_method(Class_Draw, "inspect", Draw_inspect, 0);
+    rb_define_method(Class_Draw, "marshal_dump", Draw_marshal_dump, 0);
+    rb_define_method(Class_Draw, "marshal_load", Draw_marshal_load, 1);
     rb_define_method(Class_Draw, "primitive", Draw_primitive, 1);
 
     /*-----------------------------------------------------------------------*/
@@ -815,6 +607,8 @@ Init_RMagick2(void)
     rb_define_method(Class_Pixel, "fcmp", Pixel_fcmp, -1);
     rb_define_method(Class_Pixel, "hash", Pixel_hash, 0);
     rb_define_method(Class_Pixel, "intensity", Pixel_intensity, 0);
+    rb_define_method(Class_Pixel, "marshal_dump", Pixel_marshal_dump, 0);
+    rb_define_method(Class_Pixel, "marshal_load", Pixel_marshal_load, 1);
     rb_define_method(Class_Pixel, "to_color", Pixel_to_color, -1);
     rb_define_method(Class_Pixel, "to_HSL", Pixel_to_HSL, 0);   // deprecated
     rb_define_method(Class_Pixel, "to_hsla", Pixel_to_hsla, 0);
@@ -880,6 +674,7 @@ Init_RMagick2(void)
     DCL_ATTR_ACCESSOR(Info, depth)
     DCL_ATTR_ACCESSOR(Info, dispose)
     DCL_ATTR_ACCESSOR(Info, dither)
+    DCL_ATTR_ACCESSOR(Info, endian)
     DCL_ATTR_ACCESSOR(Info, extract)
     DCL_ATTR_ACCESSOR(Info, filename)
     DCL_ATTR_ACCESSOR(Info, fill)
@@ -908,6 +703,7 @@ Init_RMagick2(void)
     DCL_ATTR_ACCESSOR(Info, stroke_width)
     DCL_ATTR_WRITER(Info, texture)
     DCL_ATTR_ACCESSOR(Info, tile_offset)
+    DCL_ATTR_ACCESSOR(Info, transparent_color)
     DCL_ATTR_ACCESSOR(Info, undercolor)
     DCL_ATTR_ACCESSOR(Info, units)
     DCL_ATTR_ACCESSOR(Info, view)
@@ -968,12 +764,15 @@ Init_RMagick2(void)
 
     /*-----------------------------------------------------------------------*/
     /* Class Magick::ImageMagickError < StandardError                        */
+    /* Class Magick::FatalImageMagickError < StandardError                   */
     /*-----------------------------------------------------------------------*/
 
     Class_ImageMagickError = rb_define_class_under(Module_Magick, "ImageMagickError", rb_eStandardError);
     rb_define_method(Class_ImageMagickError, "initialize", ImageMagickError_initialize, -1);
     rb_define_attr(Class_ImageMagickError, MAGICK_LOC, True, False);
 
+    Class_FatalImageMagickError = rb_define_class_under(Module_Magick, "FatalImageMagickError", rb_eStandardError);
+
 
     /*-----------------------------------------------------------------------*/
     /* Class Magick::DestroyedImageError < StandardError                     */
@@ -1015,15 +814,23 @@ Init_RMagick2(void)
     END_ENUM
 
     // AlphaChannelType constants
-#if defined(HAVE_TYPE_ALPHACHANNELTYPE)
     DEF_ENUM(AlphaChannelType)
         ENUMERATOR(UndefinedAlphaChannel)
         ENUMERATOR(ActivateAlphaChannel)
         ENUMERATOR(DeactivateAlphaChannel)
-        ENUMERATOR(ResetAlphaChannel)
+        ENUMERATOR(ResetAlphaChannel)  /* deprecated */
         ENUMERATOR(SetAlphaChannel)
-    END_ENUM
+#if defined(HAVE_ENUM_COPYALPHACHANNEL)
+        ENUMERATOR(CopyAlphaChannel)
+        ENUMERATOR(ExtractAlphaChannel)
+        ENUMERATOR(OpaqueAlphaChannel)
+        ENUMERATOR(ShapeAlphaChannel)
+        ENUMERATOR(TransparentAlphaChannel)
 #endif
+#if defined(HAVE_ENUM_BACKGROUNDALPHACHANNEL)
+        ENUMERATOR(BackgroundAlphaChannel)
+#endif
+    END_ENUM
 
     // AnchorType constants (for Draw#text_anchor - these are not defined by ImageMagick)
     DEF_ENUM(AnchorType)
@@ -1108,9 +915,7 @@ Init_RMagick2(void)
         ENUMERATOR(Rec709LumaColorspace)
         ENUMERATOR(Rec709YCbCrColorspace)
         ENUMERATOR(LogColorspace)
-#if defined(HAVE_ENUM_CMYCOLORSPACE)
         ENUMERATOR(CMYColorspace)
-#endif
     END_ENUM
 
     // ComplianceType constants are defined as enums but used as bit flags
@@ -1136,10 +941,11 @@ Init_RMagick2(void)
         ENUMERATOR(AddCompositeOp)
         ENUMERATOR(AtopCompositeOp)
         ENUMERATOR(BlendCompositeOp)
-        ENUMERATOR(BumpmapCompositeOp)
-#if defined(HAVE_ENUM_CHANGEMASKCOMPOSITEOP)
-        ENUMERATOR(ChangeMaskCompositeOp)
+#if defined(HAVE_ENUM_BLURCOMPOSITEOP)
+        ENUMERATOR(BlurCompositeOp)
 #endif
+        ENUMERATOR(BumpmapCompositeOp)
+        ENUMERATOR(ChangeMaskCompositeOp)
         ENUMERATOR(ClearCompositeOp)
         ENUMERATOR(ColorBurnCompositeOp)
         ENUMERATOR(ColorDodgeCompositeOp)
@@ -1154,9 +960,10 @@ Init_RMagick2(void)
         ENUMERATOR(CopyRedCompositeOp)
         ENUMERATOR(CopyYellowCompositeOp)
         ENUMERATOR(DarkenCompositeOp)
-#if defined(HAVE_ENUM_DIVIDECOMPOSITEOP)
-        ENUMERATOR(DivideCompositeOp)
+#if defined(HAVE_ENUM_DISTORTCOMPOSITEOP)
+        ENUMERATOR(DistortCompositeOp)
 #endif
+        ENUMERATOR(DivideCompositeOp)
         ENUMERATOR(DstAtopCompositeOp)
         ENUMERATOR(DstCompositeOp)
         ENUMERATOR(DstInCompositeOp)
@@ -1170,9 +977,7 @@ Init_RMagick2(void)
         ENUMERATOR(HueCompositeOp)
         ENUMERATOR(InCompositeOp)
         ENUMERATOR(LightenCompositeOp)
-#if defined(HAVE_ENUM_LINEARLIGHTCOMPOSITEOP)
         ENUMERATOR(LinearLightCompositeOp)
-#endif
         ENUMERATOR(LuminizeCompositeOp)
         ENUMERATOR(MinusCompositeOp)
         ENUMERATOR(ModulateCompositeOp)
@@ -1235,21 +1040,50 @@ Init_RMagick2(void)
         ENUMERATOR(PreviousDispose)
     END_ENUM
 
-#if defined(HAVE_DISTORTIMAGE)
     // DistortImage "method" argument values
     DEF_ENUM(DistortImageMethod)
         ENUMERATOR(UndefinedDistortion)
         ENUMERATOR(AffineDistortion)
         ENUMERATOR(AffineProjectionDistortion)
-#if defined(HAVE_ENUM_ARCDISTORTION)
         ENUMERATOR(ArcDistortion)
+#if defined(HAVE_ENUM_POLARDISTORTION)
+        ENUMERATOR(PolarDistortion)
+#endif
+#if defined(HAVE_ENUM_DEPOLARDISTORTION)
+        ENUMERATOR(DePolarDistortion)
+#endif
+#if defined(HAVE_ENUM_BARRELDISTORTION)
+        ENUMERATOR(BarrelDistortion)
 #endif
         ENUMERATOR(BilinearDistortion)
+#if defined(HAVE_ENUM_BILINEARFORWARDDISTORTION)
+        ENUMERATOR(BilinearForwardDistortion)
+#endif
+#if defined(HAVE_ENUM_BILINEARREVERSEDISTORTION)
+        ENUMERATOR(BilinearReverseDistortion)
+#endif
         ENUMERATOR(PerspectiveDistortion)
-#if defined(HAVE_PERSPECTIVEPROJECTIONDISTORTION)
         ENUMERATOR(PerspectiveProjectionDistortion)
+#if defined(HAVE_ENUM_POLYNOMIALDISTORTION)
+        ENUMERATOR(PolynomialDistortion)
 #endif
         ENUMERATOR(ScaleRotateTranslateDistortion)
+#if defined(HAVE_ENUM_SHEPARDSDISTORTION)
+        ENUMERATOR(ShepardsDistortion)
+#endif
+#if defined(HAVE_ENUM_BARRELINVERSEDISTORTION)
+        ENUMERATOR(BarrelInverseDistortion)
+#endif
+    END_ENUM
+
+#if defined(HAVE_TYPE_DITHERMETHOD)
+    DEF_ENUM(DitherMethod)
+        ENUMERATOR(UndefinedDitherMethod)
+#if defined(HAVE_ENUM_NODITHERMETHOD)
+        ENUMERATOR(NoDitherMethod)
+#endif
+        ENUMERATOR(RiemersmaDitherMethod)
+        ENUMERATOR(FloydSteinbergDitherMethod)
     END_ENUM
 #endif
 
@@ -1336,15 +1170,9 @@ Init_RMagick2(void)
         ENUMERATOR(LineInterlace)
         ENUMERATOR(PlaneInterlace)
         ENUMERATOR(PartitionInterlace)
-#if defined(HAVE_ENUM_GIFINTERLACE)
         ENUMERATOR(GIFInterlace)
-#endif
-#if defined(HAVE_ENUM_JPEGINTERLACE)
         ENUMERATOR(JPEGInterlace)
-#endif
-#if defined(HAVE_ENUM_PNGINTERLACE)
         ENUMERATOR(PNGInterlace)
-#endif
     END_ENUM
 
     DEF_ENUM(InterpolatePixelMethod)
@@ -1374,21 +1202,13 @@ Init_RMagick2(void)
         ENUMERATOR(OptimizePlusLayer)
         ENUMERATOR(CoalesceLayer)
         ENUMERATOR(DisposeLayer)
-#if defined(HAVE_ENUM_OPTIMIZETRANSLAYER)
         ENUMERATOR(OptimizeTransLayer)
-#endif
 #if defined(HAVE_ENUM_OPTIMIZEIMAGELAYER)
         ENUMERATOR(OptimizeImageLayer)
 #endif
-#if defined(HAVE_ENUM_REMOVEDUPSLAYER)
         ENUMERATOR(RemoveDupsLayer)
-#endif
-#if defined(HAVE_ENUM_REMOVEZEROLAYER)
         ENUMERATOR(RemoveZeroLayer)
-#endif
-#if defined(HAVE_ENUM_COMPOSITELAYER)
         ENUMERATOR(CompositeLayer)
-#endif
 #if defined(HAVE_ENUM_MERGELAYER)
         ENUMERATOR(MergeLayer)
 #endif
@@ -1397,6 +1217,9 @@ Init_RMagick2(void)
 #endif
 #if defined(HAVE_ENUM_FLATTENLAYER)
         ENUMERATOR(FlattenLayer)
+#endif
+#if defined(HAVE_ENUM_TRIMBOUNDSLAYER)
+        ENUMERATOR(TrimBoundsLayer)
 #endif
     END_ENUM
 
@@ -1404,9 +1227,7 @@ Init_RMagick2(void)
         ENUMERATOR(UndefinedMetric)
         ENUMERATOR(AbsoluteErrorMetric)
         ENUMERATOR(MeanAbsoluteErrorMetric)
-#if defined(HAVE_ENUM_MEANERRORPERPIXELMETRIC)
         ENUMERATOR(MeanErrorPerPixelMetric)
-#endif
         ENUMERATOR(MeanSquaredErrorMetric)
         ENUMERATOR(PeakAbsoluteErrorMetric)
         ENUMERATOR(PeakSignalToNoiseRatioMetric)
@@ -1421,9 +1242,7 @@ Init_RMagick2(void)
         ENUMERATOR(ImpulseNoise)
         ENUMERATOR(LaplacianNoise)
         ENUMERATOR(PoissonNoise)
-#if defined(HAVE_ENUM_RANDOMNOISE)
         ENUMERATOR(RandomNoise)
-#endif
     END_ENUM
 
     // Orientation constants
@@ -1497,6 +1316,45 @@ Init_RMagick2(void)
         ENUMERATOR(XorQuantumOperator)
 #if defined(HAVE_ENUM_POWEVALUATEOPERATOR)
         ENUMERATOR(PowQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_LOGEVALUATEOPERATOR)
+        ENUMERATOR(LogQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_THRESHOLDEVALUATEOPERATOR)
+        ENUMERATOR(ThresholdQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_THRESHOLDBLACKEVALUATEOPERATOR)
+        ENUMERATOR(ThresholdBlackQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_THRESHOLDWHITEEVALUATEOPERATOR)
+        ENUMERATOR(ThresholdWhiteQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_GAUSSIANNOISEEVALUATEOPERATOR)
+        ENUMERATOR(GaussianNoiseQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_IMPULSENOISEEVALUATEOPERATOR)
+        ENUMERATOR(ImpulseNoiseQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_LAPLACIANNOISEEVALUATEOPERATOR)
+        ENUMERATOR(LaplacianNoiseQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_MULTIPLICATIVENOISEEVALUATEOPERATOR)
+        ENUMERATOR(MultiplicativeNoiseQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_POISSONNOISEEVALUATEOPERATOR)
+        ENUMERATOR(PoissonNoiseQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_UNIFORMNOISEEVALUATEOPERATOR)
+        ENUMERATOR(UniformNoiseQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_COSINEEVALUATEOPERATOR)
+        ENUMERATOR(CosineQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_SINEEVALUATEOPERATOR)
+        ENUMERATOR(SineQuantumOperator)
+#endif
+#if defined(HAVE_ENUM_ADDMODULUSEVALUATEOPERATOR)
+        ENUMERATOR(AddModulusQuantumOperator)
 #endif
     END_ENUM
 
@@ -1516,6 +1374,25 @@ Init_RMagick2(void)
         ENUMERATOR(PixelsPerCentimeterResolution)
     END_ENUM
 
+#if defined(HAVE_SPARSECOLORIMAGE)
+    DEF_ENUM(SparseColorMethod)
+        ENUMERATOR(UndefinedColorInterpolate)
+        ENUMERATOR(BarycentricColorInterpolate)
+        ENUMERATOR(BilinearColorInterpolate)
+        //ENUMERATOR(PolynomialColorInterpolate)
+        ENUMERATOR(ShepardsColorInterpolate)
+        ENUMERATOR(VoronoiColorInterpolate)
+    END_ENUM
+#endif
+
+    // SpreadMethod
+    DEF_ENUM(SpreadMethod)
+        ENUMERATOR(UndefinedSpread)
+        ENUMERATOR(PadSpread)
+        ENUMERATOR(ReflectSpread)
+        ENUMERATOR(RepeatSpread)
+    END_ENUM
+
     // StorageType
     DEF_ENUM(StorageType)
         ENUMERATOR(UndefinedPixel)
@@ -1561,20 +1438,26 @@ Init_RMagick2(void)
         ENUMERATOR(DitherVirtualPixelMethod)
         ENUMERATOR(RandomVirtualPixelMethod)
         ENUMERATOR(ConstantVirtualPixelMethod)
-#if defined(HAVE_ENUM_MASKVIRTUALPIXELMETHOD)
         ENUMERATOR(MaskVirtualPixelMethod)
-#endif
-#if defined(HAVE_ENUM_BLACKVIRTUALPIXELMETHOD)
         ENUMERATOR(BlackVirtualPixelMethod)
-#endif
-#if defined(HAVE_ENUM_GRAYVIRTUALPIXELMETHOD)
         ENUMERATOR(GrayVirtualPixelMethod)
-#endif
-#if defined(HAVE_ENUM_WHITEVIRTUALPIXELMETHOD)
         ENUMERATOR(WhiteVirtualPixelMethod)
+#if defined(HAVE_ENUM_HORIZONTALTILEVIRTUALPIXELMETHOD)
+        ENUMERATOR(HorizontalTileVirtualPixelMethod)
+#endif
+#if defined(HAVE_ENUM_VERTICALTILEVIRTUALPIXELMETHOD)
+        ENUMERATOR(VerticalTileVirtualPixelMethod)
+#endif
+#if defined(HAVE_ENUM_HORIZONTALTILEEDGEVIRTUALPIXELMETHOD)
+        ENUMERATOR(HorizontalTileEdgeVirtualPixelMethod)
+#endif
+#if defined(HAVE_ENUM_VERTICALTILEEDGEVIRTUALPIXELMETHOD)
+        ENUMERATOR(VerticalTileEdgeVirtualPixelMethod)
+#endif
+#if defined(HAVE_ENUM_CHECKERTILEVIRTUALPIXELMETHOD)
+        ENUMERATOR(CheckerTileVirtualPixelMethod)
 #endif
     END_ENUM
-
     // WeightType constants
     DEF_ENUM(WeightType)
         ENUMERATOR(AnyWeight)
@@ -1644,6 +1527,14 @@ Init_RMagick2(void)
     rb_define_const(Module_Magick, "TypeMetric", Class_TypeMetric);
 
 
+    /*-----------------------------------------------------------------------*/
+    /* Error handlers                                                        */
+    /*-----------------------------------------------------------------------*/
+
+    SetFatalErrorHandler(rm_fatal_error_handler);
+    SetErrorHandler(rm_error_handler);
+    SetWarningHandler(rm_warning_handler);
+
 }
 
 
@@ -1656,7 +1547,8 @@ Init_RMagick2(void)
  *  Notes:      Bypass the test by defining the constant RMAGICK_BYPASS_VERSION_TEST
  *              to 'true' at the top level, before requiring 'RMagick'
 */
-static void test_Magick_version(void)
+static void
+test_Magick_version(void)
 {
     unsigned long version_number;
     const char *version_str;
@@ -1682,7 +1574,7 @@ static void test_Magick_version(void)
         }
 
         rb_raise(rb_eRuntimeError,
-                 "This version of RMagick was created to run with %s %s but %.*s is in use.\n" ,
+                 "This installation of RMagick was configured with %s %s but %.*s is in use.\n" ,
                  MagickPackageName, MagickLibVersionText, x, version_str);
     }
 
@@ -1696,7 +1588,8 @@ static void test_Magick_version(void)
     Static:     version_constants
     Purpose:    create Version, Magick_version, and Version_long constants.
 */
-static void version_constants(void)
+static void
+version_constants(void)
 {
     const char *mgk_version;
     volatile VALUE str;
@@ -1713,7 +1606,7 @@ static void version_constants(void)
     rb_define_const(Module_Magick, "Version", str);
 
     sprintf(long_version,
-            "This is %s ($Date: 2008/07/13 21:18:28 $) Copyright (C) 2008 by Timothy P. Hunter\n"
+            "This is %s ($Date: 2009/06/19 22:07:05 $) Copyright (C) 2009 by Timothy P. Hunter\n"
             "Built with %s\n"
             "Built for %s\n"
             "Web page: http://rmagick.rubyforge.org\n"
